import type { Hash } from "@siteimprove/alfa-hash";
import { Parser } from "@siteimprove/alfa-parser";
import type { Slice } from "@siteimprove/alfa-slice";
import { Result } from "@siteimprove/alfa-result";
import type * as json from "@siteimprove/alfa-json";
import { Token } from "../../syntax/index.js";
import type { Unit } from "../../unit/index.js";
import type { Numeric } from "../numeric/index.js";
import { Angle, Length, Number, Percentage } from "../numeric/index.js";
import type { Expression } from "./expression.js";
import type { Kind } from "./kind.js";
/**
 * {@link https://drafts.csswg.org/css-values/#math}
 *
 * @public
 */
export declare class Math<out D extends Math.Dimension = Math.Dimension> {
    static of(expression: Expression): Math;
    private readonly _expression;
    private readonly _type;
    private constructor();
    get expression(): Expression;
    get type(): "math expression";
    reduce(resolver: Expression.Resolver): Math;
    /**
     * {@link https://drafts.css-houdini.org/css-typed-om/#cssnumericvalue-match}
     */
    isDimension<D extends Numeric.Dimension>(dimension: D): this is Math<D>;
    /**
     * {@link https://drafts.css-houdini.org/css-typed-om/#cssnumericvalue-match}
     */
    isDimensionPercentage<D extends Numeric.Dimension>(dimension: D): this is Math<`${D}-percentage`>;
    /**
     * {@link https://drafts.css-houdini.org/css-typed-om/#cssnumericvalue-match}
     */
    isNumber(): this is Math<"number">;
    /**
     * {@link https://drafts.css-houdini.org/css-typed-om/#cssnumericvalue-match}
     */
    isPercentage(): this is Math<"percentage">;
    /**
     * Resolves a calculation typed as an angle, length, length-percentage or
     * number. Needs a resolver to handle relative lengths and percentages.
     */
    resolve(this: Math<"angle">, resolver?: Expression.GenericResolver): Result<Angle<"deg">, string>;
    resolve(this: Math<"angle-percentage">, resolver?: Expression.PercentageResolver<Angle<Unit.Angle.Canonical>> & Expression.GenericResolver): Result<Angle<Unit.Angle.Canonical>, string>;
    resolve(this: Math<"length">, resolver: Expression.LengthResolver & Expression.GenericResolver): Result<Length<Unit.Length.Canonical>, string>;
    resolve(this: Math<"length-percentage">, resolver: Expression.Resolver<Unit.Length.Canonical, Length<Unit.Length.Canonical>> & Expression.GenericResolver): Result<Length<Unit.Length.Canonical>, string>;
    resolve(this: Math<"number">, resolver?: Expression.GenericResolver): Result<Number, string>;
    resolve<T extends Numeric = Percentage>(this: Math<"percentage">, resolver?: Expression.PercentageResolver<T> & Expression.GenericResolver): Result<T, string>;
    hash(hash: Hash): void;
    equals(value: unknown): value is this;
    toJSON(): Math.JSON;
    toString(): string;
}
/**
 * @public
 */
export declare namespace Math {
    interface JSON {
        [key: string]: json.JSON;
        type: "math expression";
        expression: Expression.JSON;
    }
    function isCalculation(value: unknown): value is Math;
    function isNumber(value: unknown): value is Math<"number">;
    function isPercentage(value: unknown): value is Math<"percentage">;
    /**
     * @internal
     */
    type Dimension = Kind.Base | `${Numeric.Dimension}-percentage` | "number";
    const parse: Parser<Slice<Token>, Math<Dimension>, string, []>;
    const parseAngle: Parser<Slice<Token>, Math<"angle">, string, []>;
    const parseAnglePercentage: Parser<Slice<Token>, Math<"angle-percentage">, string, []>;
    const parseLength: Parser<Slice<Token>, Math<"length">, string, []>;
    const parseLengthPercentage: Parser<Slice<Token>, Math<"length-percentage">, string, []>;
    const parseNumber: Parser<Slice<Token>, Math<"number">, string, []>;
    const parsePercentage: Parser<Slice<Token>, Math<"percentage">, string, []>;
}
//# sourceMappingURL=math.d.ts.map