import type { Unit } from "./unit.js";
/**
 * Convertible units are sets of units where each unit within a given set can be
 * converted to another unit within the same set. With the exception of relative
 * lengths, all units defined in CSS are convertible.
 *
 * @public
 */
export interface Convertible<U extends string> {
    withUnit<V extends U>(unit: V): Convertible<V>;
}
/**
 * Converters are functions that convert an input value from one unit within a
 * set of units to another unit within the same set of units.
 *
 * @public
 */
export type Converter<U extends string> = (value: number, from: U, to: U) => number;
/**
 * @public
 */
export declare namespace Converter {
    /**
     * {@link https://drafts.csswg.org/css-values/#lengths}
     */
    const length: Converter<Unit.Length.Absolute>;
    /**
     * {@link https://drafts.csswg.org/css-values/#angles}
     */
    const angle: Converter<Unit.Angle>;
    /**
     * {@link https://drafts.csswg.org/css-values/#time}
     */
    const time: Converter<Unit.Time>;
    /**
     * {@link https://drafts.csswg.org/css-values/#frequency}
     */
    const frequency: Converter<Unit.Frequency>;
}
//# sourceMappingURL=converter.d.ts.map