import type { Hash } from "@siteimprove/alfa-hash";
import { type Parser as CSSParser } from "../../syntax/index.js";
import { Number, Percentage } from "../numeric/index.js";
import { Format } from "./format.js";
type ToCanonical<T extends Number | Percentage<"percentage">> = T extends Number ? Number.Canonical : T extends Percentage ? Percentage.Canonical : Number.Canonical | Percentage.Canonical;
/**
 * @public
 */
export declare class RGB<C extends Number.Canonical | Percentage.Canonical = Number.Canonical | Percentage.Fixed<"percentage">, A extends Number.Canonical | Percentage.Canonical = Number.Canonical | Percentage.Fixed<"percentage">> extends Format<"rgb"> {
    static of<C extends Number.Canonical | Percentage.Canonical, A extends Number.Canonical | Percentage.Canonical>(red: C, green: C, blue: C, alpha: A): RGB<C, A>;
    static of<C extends Number | Percentage<"percentage">, A extends Number | Percentage<"percentage">>(red: C, green: C, blue: C, alpha: A): RGB<ToCanonical<C>, ToCanonical<A>>;
    private readonly _red;
    private readonly _green;
    private readonly _blue;
    private readonly _alpha;
    private constructor();
    get red(): C;
    get green(): C;
    get blue(): C;
    get alpha(): A;
    resolve(): RGB.Canonical;
    equals(value: unknown): value is this;
    hash(hash: Hash): void;
    toJSON(): RGB.JSON;
    toString(): string;
}
/**
 * @public
 */
export declare namespace RGB {
    type Canonical = RGB<Percentage.Canonical, Percentage.Canonical>;
    interface JSON extends Format.JSON<"rgb"> {
        red: Number.Fixed.JSON | Percentage.Fixed.JSON;
        green: Number.Fixed.JSON | Percentage.Fixed.JSON;
        blue: Number.Fixed.JSON | Percentage.Fixed.JSON;
        alpha: Number.Fixed.JSON | Percentage.Fixed.JSON;
    }
    function isRGB<C extends Number.Canonical | Percentage.Canonical, A extends Number.Canonical | Percentage.Canonical>(value: unknown): value is RGB<C, A>;
    /**
     * {@link https://drafts.csswg.org/css-color/#funcdef-rgb}
     */
    const parse: CSSParser<RGB>;
}
export {};
//# sourceMappingURL=rgb.d.ts.map